<?php

namespace Moneris\Checkout\Controller;

use Moneris\Checkout\Gateway;
use Moneris\Checkout\Ticket;

class Email {
    public function __construct() {
        $this->load_hooks();
    }

    public function load_hooks() {
        add_action( 'woocommerce_email_order_meta', [ $this, 'add_moneris_meta_to_email' ], 10, 4 );
        add_filter( 'woocommerce_get_order_item_totals', [ $this, 'woocommerce_get_order_item_totals' ], 10, 2 );
        add_action( 'woocommerce_thankyou_' . Gateway::PAYMENT_CODE, [
            $this,
            'add_moneris_meta_to_confirmation'
        ], 10, 1 );
    }

    public function woocommerce_get_order_item_totals($total_rows, $order) {
      if ($order->get_payment_method() !== Gateway::PAYMENT_CODE) {
        return $total_rows;
      }

      if (empty($order->get_id())) {
          return $total_rows;
      }

      $receipt = get_post_meta( $order->get_id(), 'moneris_receipt_object', true );

      if (empty($receipt)) {
        return $total_rows;
      }

      if (isset( $receipt["response"]["receipt"]["cc"]["reference_no"] ) &&
      !empty($receipt["response"]["receipt"]["cc"]["reference_no"] )
      ) {
        $total_rows['payment_method']['value'] = __('Credit Card');
      }

      return $total_rows;
    }

    public function add_moneris_meta_to_confirmation( $order_id ) {
        $order      = wc_get_order( $order_id );
        $text_align = is_rtl() ? 'right' : 'left';
        $receipt    = get_post_meta( $order->get_id(), 'moneris_receipt_object', true );
        ?>
      <h2><?php echo __( 'Payment Information' ); ?></h2>
      <div style="margin-bottom: 40px;">
        <table class="td" cellspacing="0" cellpadding="6"
               style="width: 100%; font-family: 'Helvetica Neue', Helvetica, Roboto, Arial, sans-serif;" border="1">
          <tr>
            <td class="td" scope="col"
                style="text-align:<?php echo esc_attr( $text_align ); ?>;"><?php esc_html_e( 'Transaction Date', 'woocommerce' ); ?></td>
            <td class="td" scope="col"
                style="text-align:right"><?php esc_html_e( $receipt["response"]["receipt"]["cc"]["transaction_date_time"] ); ?></td>
          </tr>
          <tr>
            <td class="td" scope="col"
                style="text-align:<?php echo esc_attr( $text_align ); ?>;"><?php esc_html_e( 'Transaction Number', 'woocommerce' ); ?></td>
            <td class="td" scope="col"
                style="text-align:right"><?php esc_html_e( $receipt["response"]["receipt"]["cc"]["transaction_no"] ); ?></td>
          </tr>
          <tr>
            <td class="td" scope="col"
                style="text-align:<?php echo esc_attr( $text_align ); ?>;"><?php esc_html_e( 'Reference Number', 'woocommerce' ); ?></td>
            <td class="td" scope="col"
                style="text-align:right"><?php esc_html_e( $receipt["response"]["receipt"]["cc"]["order_no"] ); ?></td>
          </tr>

        </table>
      </div>
        <?php
    }

    public function add_moneris_meta_to_email( $order, $sent_to_admin, $plain_text, $email ) {
        if ( $order->get_payment_method() !== Gateway::PAYMENT_CODE ) {
            return;
        }

        $text_align = is_rtl() ? 'right' : 'left';
        $receipt    = get_post_meta( $order->id, 'moneris_receipt_object', true );
        ?>
      <h2><?php echo __( 'Payment Information' ); ?></h2>
      <div style="margin-bottom: 40px;">
        <table class="td" cellspacing="0" cellpadding="6"
               style="width: 100%; font-family: 'Helvetica Neue', Helvetica, Roboto, Arial, sans-serif;" border="1">
          <tr>
            <td class="td" scope="col"
                style="text-align:<?php echo esc_attr( $text_align ); ?>;"><?php esc_html_e( 'Transaction Date', 'woocommerce' ); ?></td>
            <td class="td" scope="col"
                style="text-align:right"><?php esc_html_e( $receipt["response"]["receipt"]["cc"]["transaction_date_time"] ); ?></td>
          </tr>
          <tr>
            <td class="td" scope="col"
                style="text-align:<?php echo esc_attr( $text_align ); ?>;"><?php esc_html_e( 'Transaction Number', 'woocommerce' ); ?></td>
            <td class="td" scope="col"
                style="text-align:right"><?php esc_html_e( $receipt["response"]["receipt"]["cc"]["transaction_no"] ); ?></td>
          </tr>
          <tr>
            <td class="td" scope="col"
                style="text-align:<?php echo esc_attr( $text_align ); ?>;"><?php esc_html_e( 'Reference Number', 'woocommerce' ); ?></td>
            <td class="td" scope="col"
                style="text-align:right"><?php esc_html_e( $receipt["response"]["receipt"]["cc"]["order_no"] ); ?></td>
          </tr>

        </table>
      </div>
        <?php
        return;
    }
}
