<?php

namespace Moneris\Checkout;

use GuzzleHttp\Client;
use Moneris\Checkout\Helper\Data;

class Ticket
{
    const PREREQUEST_ENDPOINT = 'https://gatewayt.moneris.com/chkt/request/request.php';
    const PREREQUEST_ENDPOINT_PROD = 'https://gateway.moneris.com/chkt/request/request.php';
    const MAX_CUST_ID_FIELD = 50;

    /** @var Data */
    private $data;

    public function __construct()
    {
        $this->data = new Data();
    }

    public function getEndpoint($test = true)
    {
    	if ($test) {
			return self::PREREQUEST_ENDPOINT;
		} else {
    		return self::PREREQUEST_ENDPOINT_PROD;
		}
    }

    private function formatPrice($number)
    {
        return number_format((float)$number, 2, '.', '');
    }

    public function getTicket()
    {
        $woocommerce = WC();
        /** @var \WC_Cart $cart */
        $cart = $woocommerce->cart;
        $wc_order = new \WC_Order();
        $wc_order->set_status('pending');
        $wc_order->set_cart_hash($cart->get_cart_hash());
        $wc_order->save();
        $wc_order_number = $wc_order->get_order_number();
        $woocommerce->session->set('order_awaiting_payment', $wc_order_number);
        $woocommerce->cart->calculate_totals();
        $shipping_cost = floatval($cart->get_shipping_total());
        $customer = $cart->get_customer();
        $url = $this->getEndpoint($this->data->isTestMode());
        $ship_methods = $woocommerce->session->get( 'chosen_shipping_methods' );

        if (is_array($ship_methods)) {
            $ship_methods = implode(' ', $ship_methods);
        }

        if (stripos($ship_methods, 'local_pickup') === false) {
            $is_pickup = false;
        } else {
            $is_pickup = true;
        }

        /** @var Client $client */
        $client = new Client([
            'headers' => ['Content-Type' => 'application/json']
        ]);

        // Get quote
        $quoteId = (WC()->cart->get_cart_hash());

        $requestData = new \stdClass;

        if ($is_pickup !== true) {
            $requestData->shipping_details              = new \stdClass();
            $requestData->shipping_details->address_1   = $customer->get_shipping_address_1();
            $requestData->shipping_details->address_2   = $customer->get_shipping_address_2();
            $requestData->shipping_details->city        = $customer->get_shipping_city();
            $requestData->shipping_details->province    = $customer->get_shipping_state();
            $requestData->shipping_details->country     = $customer->get_shipping_country();
            $requestData->shipping_details->postal_code = $customer->get_shipping_postcode();
        }

        {
            $requestData->billing_details              = new \stdClass();
            $requestData->billing_details->address_1   = $customer->get_billing_address_1();
            $requestData->billing_details->address_2   = $customer->get_billing_address_2();
            $requestData->billing_details->city        = $customer->get_billing_city();
            $requestData->billing_details->province    = $customer->get_billing_state();
            $requestData->billing_details->country     = $customer->get_billing_country();
            $requestData->billing_details->postal_code = $customer->get_billing_postcode();
        }

        $requestData->contact_details = new \stdClass();
        $requestData->contact_details->first_name = wc_clean($_POST['billing_first_name']);
        $requestData->contact_details->last_name = wc_clean($_POST['billing_last_name']);
        $requestData->contact_details->email = wc_clean($_POST['billing_email']);
        $requestData->contact_details->phone = wc_clean($_POST['billing_phone']);

        if (empty($customer->get_email())) {
            $customer->set_email(wc_clean($_POST['billing_email']));
        }

        $requestData->store_id = $this->data->getStoreId();
        $requestData->api_token = $this->data->getApiToken();
        $requestData->checkout_id = $this->data->getCheckoutId();
        $requestData->integrator = "cr_dev";
        $requestData->txn_total = $this->formatPrice(WC()->cart->get_total(false));
        $requestData->environment = $this->data->getMode();
        $requestData->action = "preload";
        $requestData->order_no = $wc_order_number . '_' . (string)time();
        $requestData->cust_id = $wc_order_number;
        $requestData->dynamic_descripto = "dyndesc";
        $requestData->cart = new \stdClass;
        $requestData->cart->items = [];

        $requestData->cust_id = substr(
            $requestData->cust_id . '+'  . $requestData->contact_details->email,
            0,
            self::MAX_CUST_ID_FIELD
        );

        $quoteItems = $woocommerce->cart->get_cart_contents();

        if (!empty($quoteItems)) {
            foreach ($quoteItems as $item) {
                $product = wc_get_product($item['product_id']);
                $itemDataToSend = new \stdClass();
                $image_id  = $product->get_image_id();
                $image_url = wp_get_attachment_image_url( $image_id, 'small' );
                $itemDataToSend->url = $image_url;

                $itemDataToSend->description = $product->get_name();
                $itemDataToSend->product_code = $product->get_sku();
                $itemDataToSend->unit_cost = $this->formatPrice($product->get_price());
                $itemDataToSend->quantity = $item['quantity'];

                $requestData->cart->items[] = $itemDataToSend;
            }

            if (!empty(floatval($shipping_cost))) {
                $itemDataToSend = new \stdClass();
                $itemDataToSend->description = (string)__('Shipping cost');
                $itemDataToSend->unit_cost = $this->formatPrice($shipping_cost);
                $itemDataToSend->quantity = 1;
                $itemDataToSend->url = MONERIS_PLUGIN_URL . 'img/shipping-icon.png';

                $requestData->cart->items[] = $itemDataToSend;
            }
        }

        $total = (float)$woocommerce->cart->get_total(false);
        $total_without_tax = $total - (float)$cart->get_total_tax();
        $tax = (float)$cart->get_total_tax();
        $requestData->cart->quote_id = crc32($quoteId);

        $requestData->subtotal = $this->formatPrice($total);
        $requestData->cart->subtotal = $total_without_tax;

        if ($tax > 0) {
            $tax_rate                            = bcdiv(
                bcmul( $tax, 100, 2 ),
                $total_without_tax );
            $tax_desc                            = 'Taxes';
            $requestData->cart->tax              = new \stdClass();
            $requestData->cart->tax->amount      = $tax;
            $requestData->cart->tax->description = $tax_desc;
            $requestData->cart->tax->rate        = $tax_rate;
        }

        if ( function_exists( 'determine_locale' ) ) {
            $locale = determine_locale();
        } else {
            $locale = is_admin() ? get_user_locale() : get_locale();
        }

        if (is_int(stripos($locale, 'fr')) && stripos($locale, 'fr') >= 0) {
            $requestData->language = 'fr';
        }

        $this->data->log(json_encode($requestData));

		$response = $client->post($url,
            ['body' => json_encode(
                $requestData
            )]
        );

		$body_content = $response->getBody()->getContents();
		$body = json_decode($body_content, true);
        $this->data->log($body);

		if ($body['response']['success'] === "true") {
            return [
                'ticket' => $body['response']['ticket'],
                'quote_id' => $requestData->order_no
            ];
        } else {
		    $error = [];

		    if (isset($body["response"]["error"]) && !empty($body["response"]["error"])) {
		        $error = array_values($body["response"]["error"]);
            }

            return [
                'result' => $body['response'],
                'error' => $error,
            ];
        }

        return null;
    }

    public function getReceiptData($ticket)
    {
        $url = $this->getEndpoint($this->data->isTestMode());

        /** @var Client $client */
        $client = new Client([
            'headers' => ['Content-Type' => 'application/json']
        ]);

        $requestData = new \stdClass;
        $requestData->store_id = $this->data->getStoreId();
        $requestData->api_token = $this->data->getApiToken();
        $requestData->checkout_id = $this->data->getCheckoutId();
        $requestData->ticket = $ticket;
        $requestData->environment = $this->data->getMode();
        $requestData->action = 'receipt';
        $this->data->log('Request receipt ' . json_encode($requestData));

        $response = $client->post($url,
            ['body' => json_encode(
                $requestData
            )]
        );

        $body = json_decode($response->getBody()->getContents(), true);

        return $body;
    }
}
